{ Demo program for ChroMathsome - Linear Algebra Pro.
  Below, you find example implementations of using functions available in
  ChroMathsome - Linear Algebra Pro.

  This code was written during the development stages of ChroMathsome.
  There's no pre-written script followed so you may find (some of) this code
  (the order of implemented functions) and some of the variables a little strange.
  
  Note that, since ChroMathsome has its own pre-defined exceptions, you do not need to
  write supersofisticated exception handling routines.
  The only exceptions written here are to make sure the pointers get freed, even
  when an exception has occured (a try ... finally block).

  Since ChroMathsome - Linear Algebra automatically updates the dimensions of
  matrices and vectors (if needed), we gave most of the matrices/vectors in the
  initialization part (InitRealMatrix, ...) a dummy value.
  You are, however, encouraged to enter the correct values (if you know them at forehand)
  since this will make your code much more readable!
  For example:
  In the eigenvalues section, we wrote
     InitRealMatrix(M, 6, 6);
     InitComplexVector(V, 1);
     InitRealVector(V1, 1);
  It's obvious that both vectors MUST be 6-dimensional.
  In this demo, we used our privilege to ignore this :-))

  When studying the code below, notice how nicely ChroMathsome - Linear Algebra
  keeps the user away from the implementation of matrices/vectors (information hiding principle).
  Matrices can be used as single datatypes.
  Example:

  (* Copy matrix M (including its dimensions) to matrix M1 -> M1 = M. *)
  Copy_RealMatrix(M1, M);
  (* Transform the last column of matrix M1 into a vector V1. *)
  RealMatrixColToRealVector(V1, M1, M1.Col);
  (* Delete the last column of matrix M1. *)
  RealMatrix_DeleteColumn(M1, M1.Col);
  (* Solve the system M1 * M = V1. *)
  SysOfEquations(M, M1, V1);

  Imagine how much code it takes to write this from scratch ...
  Also, notice that the code can (almost) replace the pseudo-code that describes what
  you want to do!

  Note that if you use matrices with large dimensions ( > 20*20), the memo (visible interface)
  will become full very fast and the program will stop giving results.
  This is a shortcoming of the memo component (Delphi 1), not of ChroMathsome - Linear Algebra.
  Eventually delete the contents of the memo when it becomes full (or write results to a file).

  Read the help file for information about the implemented functions
  and their limitations/possibilities!

  Author: John Merrit (john.merrit@net4all.be)
  Release date: April 16 1997
  Reader may do with this code whatever he/she wants.

  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  Brought to you by the letter "A".              }

unit Prodemo;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, LinAlg, LinAlgP, Menus, StdCtrls;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    MainMenu1: TMainMenu;
    exit1: TMenuItem;
    Eigenvalues1: TMenuItem;
    GeneralMatrix1: TMenuItem;
    HighMultiplicity1: TMenuItem;
    RootsofPolynomial1: TMenuItem;
    Clear1: TMenuItem;
    Decomposition1: TMenuItem;
    SingularValue1: TMenuItem;
    Systems1: TMenuItem;
    SysofEquations1: TMenuItem;
    Functions1: TMenuItem;
    Procedures1: TMenuItem;
    RealMatrix1: TMenuItem;
    ComplexMatrix1: TMenuItem;
    RealMatrix2: TMenuItem;
    ComplexMatrix2: TMenuItem;
    Determinant1: TMenuItem;
    Rank1: TMenuItem;
    CoFactor1: TMenuItem;
    Determinant2: TMenuItem;
    Rank2: TMenuItem;
    CoFactor2: TMenuItem;
    Inverse1: TMenuItem;
    Inverse2: TMenuItem;
    Notarandomrealmatrix1: TMenuItem;
    Hessenberg2: TMenuItem;
    Complexsystem1: TMenuItem;
    procedure exit1Click(Sender: TObject);
    procedure Eigenvalues(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure SingularValue1Click(Sender: TObject);
    procedure SysofEquations1Click(Sender: TObject);
    procedure MatrixFunctions(Sender: TObject);
    procedure MatrixProcedures(Sender: TObject);
    procedure Notarandomrealmatrix1Click(Sender: TObject);
    procedure Complexsystem1Click(Sender: TObject);

  private
    { Private declarations }
    procedure Write_Message(s: string);
    procedure Write_RealMatrix(M: RealMatrix);
    procedure Write_ComplexMatrix(M: ComplexMatrix);
    procedure Write_RealVector(V: RealVector);
    procedure Write_ComplexVector(V: ComplexVector);
    procedure RetrieveRealMatrix(var M: RealMatrix);
    procedure Write_MatrixToFile(M: RealMatrix; FileName: string);
    procedure Write_ComplexMatrixToFile(M: ComplexMatrix; FileName: string);
    
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}
uses MatrxDLG;

procedure TForm1.Write_Message(s: string);
begin
     Memo1.Lines.add(s);
end;

{ Write a real matrix to the screen, creating a nice readable look. }
procedure TForm1.Write_RealMatrix(M: RealMatrix);
var a,b:integer;
    matt,mats,mattt:string;
begin
    memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    if abs(M.Matrix^[a]^[b]) < 1E-15 then
    begin
    M.Matrix^[a]^[b]:= 0;
    matt:= '0';
    end
    else
    matt:= FloatToStrF(M.Matrix^[a]^[b], ffexponent,5,2);
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    memo1.Lines.add(mats);
    end;
end;

procedure TForm1.Write_ComplexMatrix(M: ComplexMatrix);
var a,b:integer;
    matt,mats,mattt:string;
    c,d: extended;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    c:= M.Matrix^[a]^[b].re;
    if abs(c) < 1E-12 then c:= 0;
    d:= M.Matrix^[a]^[b].im;
    if abs(d) < 1E-12 then d:= 0;
    matt:= '(' + FloatToStr(c) + ';' + FloatToStr(d) + ')';
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    Memo1.Lines.add(mats);
    end;
end;

procedure TForm1.Write_MatrixToFile(M: RealMatrix; FileName: string);
var a,b:integer;
    matt,mats,mattt:string;
    F: TextFile;
begin
    AssignFile(F, FileName);
    Rewrite(F);
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    if abs(M.Matrix^[a]^[b]) < 1E-10 then
    begin
    M.Matrix^[a]^[b]:= 0;
    matt:= '';
    end
    else
    matt:= FloatToStrF(M.Matrix^[a]^[b], ffexponent,5,2);
    mattt:= mattt + matt;
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    writeln(F, mats);
    end;
    CloseFile(F);
end;


{ Complex counterpart of above. }
procedure TForm1.Write_ComplexMatrixToFile(M: ComplexMatrix; FileName: string);
var a,b:integer;
    matt,mats,mattt:string;
    c,d: extended;
    F: TextFile;
begin
    AssignFile(F, FileName);
    Rewrite(F);
    mattt:= '';
    for a:= 1  to M.Row do
    begin
    mats:= '[ ';
    for b:= 1 to M.Col do
    begin
    c:= M.Matrix^[a]^[b].re;
    if abs(c) < 1E-12 then c:= 0;
    d:= M.Matrix^[a]^[b].im;
    if abs(d) < 1E-12 then d:= 0;
    if ((c = 0) and (d = 0)) then matt:= '' else
    matt:= '(' + FloatToStr(c) + ';' + FloatToStr(d) + ')';
    mattt:= mattt + matt;
    end;
    mats:= mats + mattt + ']';
    mattt:= '';
    writeln(F, mats);
    end;
    CloseFile(F);
end;

{ Write a real vector to the screen, creating a nice readable look. }
procedure TForm1.Write_RealVector(V: RealVector);
var a:integer;
    matt,mats,mattt:string;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    mats:= '[ ';
    for a:= 1 to V.Dimension do
    begin
    if abs(V.Vector^[a]) < 1E-10 then
    begin
    V.Vector^[a]:= 0;
    matt:= IntToStr(round(V.Vector^[a]));
    end
    else
    matt:= FloatToStrF(V.Vector^[a], ffexponent,5,2);
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    Memo1.Lines.add(mats);
end;

{ Complex counterpart of above. }
procedure TForm1.Write_ComplexVector(V: ComplexVector);
var a:integer;
    matt,mats,mattt:string;
begin
    Memo1.Enabled:= true;
    mattt:= '';
    mats:= '[ ';
    for a:= 1 to V.Dimension do
    begin
    matt:= '(' + FloatToStr(V.Vector^[a].re) + ';' +  FloatToStr(V.Vector^[a].im) + ')';
    mattt:= mattt + matt + ' ';
    end;
    mats:= mats + mattt + ']';
    Memo1.Lines.add(mats);
end;

procedure TForm1.exit1Click(Sender: TObject);
begin
     Close;
end;

procedure TForm1.Clear1Click(Sender: TObject);
begin
     if MessageDlg('Delete the contents of this memo?',
     mtInformation, [mbYes, mbNo], 0) = mrYes
     then
     Memo1.Lines.Clear;
end;

{ Calculate the eigenvalues and characteristic equation of a matrix.
  First, we calculate them for a general matrix;
  next, we calculate them for a matrix that has eigenvalues with a multiplicity higher than 1;
  last, we try to find the roots of a (real) polynom.                                       }
procedure TForm1.Eigenvalues(Sender: TObject);
var M: RealMatrix;
    V: ComplexVector;
    V1: RealVector;
    i: integer;
begin
     InitRealMatrix(M, 6, 6);
     InitComplexVector(V, 1);
     InitRealVector(V1, 1);

     try
     if Sender = GeneralMatrix1 then
     begin
     M.Matrix^[1]^[1]:= 5.3;
     M.Matrix^[1]^[2]:= 2.3;
     M.Matrix^[1]^[3]:= 4.6;
     M.Matrix^[1]^[4]:= 2.7;
     M.Matrix^[1]^[5]:= 1.6;
     M.Matrix^[1]^[6]:= 2.2;
     M.Matrix^[2]^[1]:= 2.4;
     M.Matrix^[2]^[2]:= 7.8;
     M.Matrix^[2]^[3]:= 5.7;
     M.Matrix^[2]^[4]:= 8.4;
     M.Matrix^[2]^[5]:= 3.4;
     M.Matrix^[2]^[6]:= 4.2;
     M.Matrix^[3]^[1]:= 3.4;
     M.Matrix^[3]^[2]:= 5.6;
     M.Matrix^[3]^[3]:= 2.4;
     M.Matrix^[3]^[4]:= 1.7;
     M.Matrix^[3]^[5]:= 7.4;
     M.Matrix^[3]^[6]:= 3.9;
     M.Matrix^[4]^[1]:= 8.3;
     M.Matrix^[4]^[2]:= 7.5;
     M.Matrix^[4]^[3]:= 9.2;
     M.Matrix^[4]^[4]:= 6.1;
     M.Matrix^[4]^[5]:= 5.2;
     M.Matrix^[4]^[6]:= 7.9;
     M.Matrix^[5]^[1]:= 4.3;
     M.Matrix^[5]^[2]:= 5.9;
     M.Matrix^[5]^[3]:= 7.2;
     M.Matrix^[5]^[4]:= 2.6;
     M.Matrix^[5]^[5]:= 4.9;
     M.Matrix^[5]^[6]:= 0.8;
     M.Matrix^[6]^[1]:= 0.9;
     M.Matrix^[6]^[2]:= 2.7;
     M.Matrix^[6]^[3]:= 4.9;
     M.Matrix^[6]^[4]:= 4.8;
     M.Matrix^[6]^[5]:= 6.7;
     M.Matrix^[6]^[6]:= 4.8;

     Write_Message('Original matrix: ');
     Write_RealMatrix(M);
     RealMatrix_Eigenvalues(V, M);
     Write_Message('Matrix eigenvalues: ');
     Write_ComplexVector(V);
     end
     { Find the eigenvalues for the matrix 5*In.
       The characteristic equation is (5 - )^n.
       In other words, all eigenvalues will be 5 with increasing multiplicity.
       Note that calculating eigenvalues with higher multiplicity is not so obvious as it
       may seem at first sight!
       The higher the multiplicity, the lower the accuracy becomes.
       See also examples lower. }
     else if Sender = HighMultiplicity1 then
     begin
     for i:= 1 to 10 do
     begin
     Redimension_RealMatrix(M, i, i);
     Redimension_ComplexVector(V, i);
     Redimension_RealVector(V1, i);
     RealIdentityMatrix(M);
     RealMatrix_RealScalarMultiplication(M, 5);
     Write_Message('Original matrix of order ' + IntToStr(i));
     {Write_RealMatrix(M);}

     RealMatrix_CharacteristicEquation(V1, M);
     Write_Message('Coefficients of characteristic equation: ');
     Write_RealVector(V1);

     Write_Message('Eigenvalues: ');
     RealMatrix_Eigenvalues(V, M);
     Write_ComplexVector(V);
     end;
     end
     else { Roots of Polynom. See help file for more theoretical background information. }
     begin
     { Polynom coefficients -> Frobenius matrix. }
     { Example 1: x^5 - 0.2*x^4 + 7*x^3 + x^2 - 3.5*x + 2 = 0. }
     Redimension_RealVector(V1, 5);
     V1.Vector^[1]:= -0.2;
     V1.Vector^[2]:= 7;
     V1.Vector^[3]:= 1;
     V1.Vector^[4]:= -3.5;
     V1.Vector^[5]:= 2;
     RealFrobeniusMatrix(M, V1);
     Write_Message('Original matrix: ');
     Write_RealMatrix(M);
     RealMatrix_Eigenvalues(V, M);
     Write_Message('Roots of polynom: ');
     Write_ComplexVector(V);

     { Example 2: x^6 - 6*x^5 + 15*x^4 - 20*x^3 + 15*x^2 - 6*x + 1 = 0.
       This example proves the remark about the accuracy, above.
       The roots of this equation are all 1 (thus with a multiplicity of 6).
       The accuracy is only 2 digits!                                        }
     Redimension_RealVector(V1, 6);
     V1.Vector^[1]:= -6;
     V1.Vector^[2]:= 15;
     V1.Vector^[3]:= -20;
     V1.Vector^[4]:= 15;
     V1.Vector^[5]:= -6;
     V1.Vector^[6]:= 1;
     RealFrobeniusMatrix(M, V1);
     Write_Message('Original matrix: ');
     Write_RealMatrix(M);
     RealMatrix_Eigenvalues(V, M);
     Write_Message('Roots of polynom: ');
     Write_ComplexVector(V);
     { Test with multiplicity only 3. }
     Redimension_RealVector(V1, 3);
     V1.Vector^[1]:= -3;
     V1.Vector^[2]:= 3;
     V1.Vector^[3]:= -1;
     RealFrobeniusMatrix(M, V1);
     Write_Message('Original matrix: ');
     Write_RealMatrix(M);
     RealMatrix_Eigenvalues(V, M);
     Write_Message('Roots of polynom: ');
     Write_ComplexVector(V);
     end;

     finally
     DestroyRealMatrix(M);
     DestroyComplexVector(V);
     DestroyRealVector(V1);
     end;
end;


{ Decomposes a m*n matrix (m >= n ) via the singular value decomposition scheme.
  M = U * W * Transpose(V) .
  U is a column-orthogonal m*n matrix;
  W is a n*n diagonal matrix (holds the singular values of M);
  Transpose(V) is a n*n orthogonal matrix.
  procedure RealMatrix_SingularValueDecomposition(M, V, W)
  returns matrix M (U in the above definition), V (V above) and a
  vector W that holds the diagonal elements of the diagonal matrix W above.
  Note that the returned matrix V is the matrix ... V NOT Transpose(V) above.
  Note also that your original matrix (M) will become the matrix U (above).
  If you wish to keep your original matrix, copy it to another first (Copy_RealMatrix(...)).}
procedure TForm1.SingularValue1Click(Sender: TObject);
var M, M1, M2, M3: RealMatrix;
    V: RealVector;
begin
     InitRealMatrix(M, 10, 10);
     InitRealMatrix(M1, 5, 5);
     InitRealMatrix(M2, 10, 10);
     InitRealMatrix(M3, 10, 10);
     InitRealVector(V, 5);
     try
     Random_RealMatrix(M, 10);
     Write_Message('Original matrix: ');
     Write_RealMatrix(M);

     RealMatrix_SingularValueDecomposition(M, M1, V);
     Write_Message('Matrix M: ');
     Write_RealMatrix(M);
     Write_Message('Matrix M1: ');
     Write_RealMatrix(M1);
     Write_Message('Coefficients of diagonal matrix V: ');
     Write_RealVector(V);

     Write_Message('Test: ');
     RealDiagonalMatrix(M2, V);
     RealMatrix_Multiplication(M3, M, M2);
     RealMatrix_Transpose(M1);
     RealMatrix_Multiplication(M, M3, M1); { M = U * W * Transpose(V) }
     Write_Message('Matrix M: ');
     Write_RealMatrix(M);

     finally
     DestroyRealMatrix(M);
     DestroyRealMatrix(M1);
     DestroyRealMatrix(M2);
     DestroyRealMatrix(M3);
     DestroyRealVector(V);
     end;
end;

{ Let's solve some sample systems of linear equations.
  Note the resulting MATRIX!
  Read the help file for information on how to interpret this matrix. }
procedure TForm1.SysofEquations1Click(Sender: TObject);
var M, CoM: RealMatrix;
    CoV: RealVector;
begin
     InitRealMatrix(M, 3, 3);
     InitRealMatrix(CoM, 3, 3);
     InitRealVector(CoV, 3);
     try
     { Example 1: Simple 3 * 3 system -> 1 unique solution. }
     CoM.Matrix^[1]^[1]:= 3;
     CoM.Matrix^[1]^[2]:= -2;
     CoM.Matrix^[1]^[3]:= 2;
     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= 2;
     CoM.Matrix^[2]^[3]:= -3;
     CoM.Matrix^[3]^[1]:= 4;
     CoM.Matrix^[3]^[2]:= 1;
     CoM.Matrix^[3]^[3]:= 2;

     CoV.Vector^[1]:= 10;
     CoV.Vector^[2]:= -1;
     CoV.Vector^[3]:= 3;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     SysOfEquations(M, CoM, CoV);
     Write_Message('Solution matrix: ');
     Write_RealMatrix(M);

     { Example 2: A 3 * 4 system -> system has infinitely many solutions of the form
               x:= x1 + *x2. }
     Redimension_RealMatrix(CoM, 3, 4);
     CoM.Matrix^[1]^[1]:= 1;
     CoM.Matrix^[1]^[2]:= 2;
     CoM.Matrix^[1]^[3]:= -1;
     CoM.Matrix^[1]^[4]:= 3;
     CoM.Matrix^[2]^[1]:= 2;
     CoM.Matrix^[2]^[2]:= 3;
     CoM.Matrix^[2]^[3]:= -2;
     CoM.Matrix^[2]^[4]:= 4;
     CoM.Matrix^[3]^[1]:= 1;
     CoM.Matrix^[3]^[2]:= 1;
     CoM.Matrix^[3]^[3]:= -1;
     CoM.Matrix^[3]^[4]:= 2;

     CoV.Vector^[1]:= 2;
     CoV.Vector^[2]:= -1;
     CoV.Vector^[3]:= -3;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     SysOfEquations(M, CoM, CoV);
     Write_Message('Solution matrix: ');
     Write_RealMatrix(M);


     { Example 3: Simple 3 * 3 system -> infinitely many solutions. }
     Redimension_RealMatrix(CoM, 3, 3);
     CoM.Matrix^[1]^[1]:= 2;
     CoM.Matrix^[1]^[2]:= 5;
     CoM.Matrix^[1]^[3]:= -3;
     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= -2;
     CoM.Matrix^[2]^[3]:= 1;
     CoM.Matrix^[3]^[1]:= 7;
     CoM.Matrix^[3]^[2]:= 4;
     CoM.Matrix^[3]^[3]:= -3;

     CoV.Vector^[1]:= 3;
     CoV.Vector^[2]:= 2;
     CoV.Vector^[3]:= 12;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     SysOfEquations(M, CoM, CoV);
     Write_Message('Solution matrix: ');
     Write_RealMatrix(M);

     { Example 4: homogeneous 3*3 system with only trivial solution (0, 0, 0). }
     Redimension_RealMatrix(CoM, 3, 3);
     CoM.Matrix^[1]^[1]:= 3;
     CoM.Matrix^[1]^[2]:= -2;
     CoM.Matrix^[1]^[3]:= 2;
     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= 2;
     CoM.Matrix^[2]^[3]:= -3;
     CoM.Matrix^[3]^[1]:= 4;
     CoM.Matrix^[3]^[2]:= 1;
     CoM.Matrix^[3]^[3]:= 2;

     CoV.Vector^[1]:= 0;
     CoV.Vector^[2]:= 0;
     CoV.Vector^[3]:= 0;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     SysOfEquations(M, CoM, CoV);
     Write_Message('Solution matrix: ');
     Write_RealMatrix(M);

     { Example 5: homogeneous 3*3 system with infinitely many solution(s) besides the
       trivial one (0, 0, 0). }
     Redimension_RealMatrix(CoM, 3, 3);
     CoM.Matrix^[1]^[1]:= 2;
     CoM.Matrix^[1]^[2]:= 5;
     CoM.Matrix^[1]^[3]:= -3;
     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= -2;
     CoM.Matrix^[2]^[3]:= 1;
     CoM.Matrix^[3]^[1]:= 7;
     CoM.Matrix^[3]^[2]:= 4;
     CoM.Matrix^[3]^[3]:= -3;

     CoV.Vector^[1]:= 0;
     CoV.Vector^[2]:= 0;
     CoV.Vector^[3]:= 0;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     SysOfEquations(M, CoM, CoV);
     Write_Message('Solution matrix: ');
     Write_RealMatrix(M);

     { Example 6: Simple 3 * 3 system -> system has no solution(s). }
     Redimension_RealMatrix(CoM, 3, 3);
     CoM.Matrix^[1]^[1]:= 2;
     CoM.Matrix^[1]^[2]:= 5;
     CoM.Matrix^[1]^[3]:= -3;
     CoM.Matrix^[2]^[1]:= 1;
     CoM.Matrix^[2]^[2]:= -2;
     CoM.Matrix^[2]^[3]:= 1;
     CoM.Matrix^[3]^[1]:= 7;
     CoM.Matrix^[3]^[2]:= 4;
     CoM.Matrix^[3]^[3]:= -3;

     CoV.Vector^[1]:= 3;
     CoV.Vector^[2]:= 2;
     CoV.Vector^[3]:= -4;
     Write_Message('System A * X = Y with: ');
     Write_Message('A: ');
     Write_RealMatrix(CoM);
     Write_Message('Y: ');
     Write_RealVector(CoV);

     SysOfEquations(M, CoM, CoV);
     Write_Message('Solution matrix: ');
     Write_RealMatrix(M);

     finally
     DestroyRealMatrix(M);
     DestroyRealMatrix(CoM);
     DestroyRealVector(CoV);
     end;
end;

procedure TForm1.MatrixFunctions(Sender: TObject);
var M: RealMatrix;
    CM: ComplexMatrix;
    i, j: integer;
    rdet: extended;
    cdet: complex;
begin
     InitRealMatrix(M, 5, 5);
     InitComplexMatrix(CM, 3, 3);
     try
     Random_RealMatrix(M, 10);
     Random_ComplexMatrix(CM, 10);
     Write_Message('Original real matrix: ');
     Write_RealMatrix(M);
     Write_Message('Original complex matrix: ');
     Write_ComplexMatrix(CM);

     if Sender = Determinant1 then
     begin
     rdet:= RealMatrix_Determinant(M);
     Write_Message('Result: ' + FloatToStr(rdet));
     end
     else if Sender = Determinant2 then
     begin
     cdet:= ComplexMatrix_Determinant(CM);
     Write_Message('Result: (' + FloatToStr(cdet.re) + ';' + FloatToStr(cdet.im) + ')');
     end
     else if ((Sender = Rank1) or (Sender = Rank2)) then
     begin
     if Sender = Rank1 then i:= RealMatrix_Rank(M)
     else i:= ComplexMatrix_Rank(CM);
     Write_Message('Result: ' + IntToStr(i));
     end
     else
     begin
     i:= random(M.Row) + 1;
     j:= random(M.Col) + 1;
     if Sender = CoFactor1 then
     begin
     rdet:= RealMatrix_CoFactor(M, i, j);
     Write_Message('Result: ' + FloatToStr(rdet));
     end
     else
     begin
     cdet:= ComplexMatrix_CoFactor(CM, i, j);
     Write_Message('Result: (' + FloatToStr(cdet.re) + ';' + FloatToStr(cdet.im) + ')');
     end;
     end;

     finally
     DestroyRealMatrix(M);
     DestroyComplexMatrix(CM);
     end;
end;

procedure TForm1.MatrixProcedures(Sender: TObject);
var M, M1, M2: RealMatrix;
    CM, CM1, CM2: ComplexMatrix;
begin
      InitRealMatrix(M, 35, 35);
      InitRealMatrix(M1, 5, 5);
      InitRealMatrix(M2, 5, 5);
      InitComplexMatrix(CM, 50, 50);
      InitComplexMatrix(CM1, 5, 5);
      InitComplexMatrix(CM2, 5, 5);
      try
      Random_RealMatrix(M, 100);
      Random_ComplexMatrix(CM, 10000);

      if Sender = Inverse1 then
      begin
      Copy_RealMatrix(M1, M);
      Write_Message('Original matrix: ');
      Write_RealMatrix(M);
      RealMatrix_Inverse(M);
      Write_Message('Result: ');
      Write_RealMatrix(M);
      Write_Message('Test: ');
      RealMatrix_Multiplication(M2, M1, M);
      Write_RealMatrix(M2);
      end
      else if Sender = Hessenberg2 then
      begin
      Write_Message('Original matrix: ');
      Write_RealMatrix(M);
      RealMatrix_HessenbergForm(M);
      Write_Message('Result: ');
      Write_RealMatrix(M);
      end
      else if Sender = Inverse2 then
      begin
      Copy_ComplexMatrix(CM1, CM);
      Write_Message('Original matrix: ');
      {Write_ComplexMatrix(CM);}
      ComplexMatrix_Inverse(CM);
      Write_Message('Result: ');
      {Write_ComplexMatrix(CM);}
      Write_Message('Test: ');
      ComplexMatrix_Multiplication(CM2, CM1, CM);
      Write_ComplexMatrixToFile(CM2, 'Identity.TXT');
      {Write_ComplexMatrix(CM2);}
      end;

      finally
      DestroyRealMatrix(M);
      DestroyRealMatrix(M1);
      DestroyRealMatrix(M2);
      DestroyComplexMatrix(CM);
      DestroyComplexMatrix(CM1);
      DestroyComplexMatrix(CM2);
      end;
end;

procedure TForm1.RetrieveRealMatrix(var M: RealMatrix);
var i, j, Row, Col: integer;
    x: extended;
begin
     Row:= MatrxDLG.EnterMatrixDlg.MatrixGrid.RowCount;
     Col:= MatrxDLG.EnterMatrixDlg.MatrixGrid.ColCount;
     Redimension_RealMatrix(M, Row - 1, Col - 1);
     for i:= 1 to M.Row do
     for j:= 1 to M.Col do
     begin
     x:= StrToFloat(MatrxDLG.EnterMatrixDlg.MatrixGrid.Cells[j,i]);
     M.Matrix^[i]^[j]:= x;
     end;
end;

{ Some of the above but now the user can enter a matrix. }
procedure TForm1.Notarandomrealmatrix1Click(Sender: TObject);
var M, M1: RealMatrix;
    det: extended;
    V1: RealVector;
    V: ComplexVector;
begin
      InitRealMatrix(M, 3, 3); { Dummy values (3's) , RetrieveRealMatrix will update dimensions. }
      InitRealMatrix(M1, 3, 3);
      InitComplexVector(V, 1); { 1 -> dummy ... }
      InitRealVector(V1, 1);
      try
      MatrxDLG.EnterMatrixDlg.MatrixGrid.RowCount:= 4;
      MatrxDLG.EnterMatrixDlg.MatrixGrid.ColCount:= 4;
      MatrxDLG.EnterMatrixDlg.Edit1.Text:= '3';
      MatrxDLG.EnterMatrixDlg.Edit2.Text:= '3';
      if MatrxDlg.EnterMatrixDlg.ShowModal = id_OK then
      begin
      RetrieveRealMatrix(M);
      Write_Message('Original matrix: ');
      Write_RealMatrix(M);

      if MatrxDLG.EnterMatrixDlg.RadioGroup1.ItemIndex = 0 then
      begin
      RealMatrix_GaussNormalForm(M);
      Write_Message('Result: ');
      Write_RealMatrix(M);
      end
      else if MatrxDLG.EnterMatrixDlg.RadioGroup1.ItemIndex = 1 then
      begin
      RealMatrix_HermiteNormalForm(M);
      Write_Message('Result: ');
      Write_RealMatrix(M);
      end
      else if MatrxDLG.EnterMatrixDlg.RadioGroup1.ItemIndex = 2 then
      begin
      RealMatrix_Inverse(M);
      Write_Message('Result: ');
      Write_RealMatrix(M);
      end
      else if MatrxDLG.EnterMatrixDlg.RadioGroup1.ItemIndex = 3 then
      begin
      det:= RealMatrix_Determinant(M);
      Write_Message('Result: ' + FloatToStr(det));
      end
      else if MatrxDLG.EnterMatrixDlg.RadioGroup1.ItemIndex = 4 then
      begin
      RealMatrix_Eigenvalues(V, M);
      Write_Message('Eigenvalues of matrix: ');
      Write_ComplexVector(V);
      end
      else if MatrxDLG.EnterMatrixDlg.RadioGroup1.ItemIndex = 5 then
      begin
      { Just enter the system A*X = Y as an augmented matrix A|Y.
        We will un-attach the Y from the matrix. }
      Copy_RealMatrix(M1, M);
      RealMatrixColToRealVector(V1, M1, M1.Col);
      RealMatrix_DeleteColumn(M1, M1.Col);
      SysOfEquations(M, M1, V1);
      Write_Message('Solution matrix: ');
      Write_RealMatrix(M);
      end;
      end;
      finally
      DestroyRealMatrix(M);
      DestroyRealMatrix(M1);
      DestroyRealVector(V1);
      DestroyComplexVector(V);
      end;
end;

{ Solve a system of linear equations with complex numbers.
  We'll use Cramer's rule to solve the system.
  Verify the result by substituting it in the original equations (it's correct, believe me ;-)).
  This code shows how easily one can extend the code from ChroMathsome - Linear Algebra
  and implement new functions.
  Note: this is only an example.
  The above systems are not solved using this "primitive" Cramer rule.
  BTW, the code below uses 2 undocumented functions.
  Get the source code (= register) and you will find more of them.          }
procedure TForm1.Complexsystem1Click(Sender: TObject);
var M, M1: ComplexMatrix;
    V: ComplexVector;
    c, c1: complex;
    i: integer;
begin
     InitComplexMatrix(M, 3, 3);
     InitComplexMatrix(M1, 3, 3);
     InitComplexVector(V, 3);
     try
     M.Matrix^[1]^[1].re:= 1;
     M.Matrix^[1]^[1].im:= 2;
     M.Matrix^[1]^[2].re:= 3;
     M.Matrix^[1]^[2].im:= 4;
     M.Matrix^[1]^[3].re:= 5;
     M.Matrix^[1]^[3].im:= 6;
     M.Matrix^[2]^[1].re:= 7;
     M.Matrix^[2]^[1].im:= 8;
     M.Matrix^[2]^[2].re:= 9;
     M.Matrix^[2]^[2].im:= 0;
     M.Matrix^[2]^[3].re:= 1;
     M.Matrix^[2]^[3].im:= 2;
     M.Matrix^[3]^[1].re:= 3;
     M.Matrix^[3]^[1].im:= 4;
     M.Matrix^[3]^[2].re:= 5;
     M.Matrix^[3]^[2].im:= 6;
     M.Matrix^[3]^[3].re:= 7;
     M.Matrix^[3]^[3].im:= 8;

     V.Vector^[1].re:= 1;
     V.Vector^[1].im:= 2;
     V.Vector^[2].re:= 3;
     V.Vector^[2].im:= 4;
     V.Vector^[3].re:= 5;
     V.Vector^[3].im:= 6;

     Write_Message('Original matrix: ');
     Write_ComplexMatrix(M);
     Write_Message('Original vector: ');
     Write_ComplexVector(V);

     Copy_ComplexMatrix(M1, M);
     c:= ComplexMatrix_Determinant(M1);

     if IsComplexZero(c) = false then  { Undocumented function ... }
     begin
     for i:= 1 to M.Col do
     begin
     Copy_ComplexMatrix(M1, M);
     ComplexMatrix_ReplaceColumn(M1, V, i);
     c1:= CDivision(ComplexMatrix_Determinant(M1), c); { Undocumented function ... }
     Write_Message('Result: (' + FloatToStr(c1.re) + ';' + FloatToStr(c1.im) + ')');
     end;
     end
     else
     Write_Message('Determinant = 0 ');

     finally
     DestroyComplexMatrix(M);
     DestroyComplexMatrix(M1);
     DestroyComplexVector(V);
     end;
end;

end.
